#!/usr/bin/env python

import os
import gconf


def set_gconf_value(value_type, key_name, value):
    """Sets a specific key in GConf"""

    try:
        client = gconf.client_get_default()

        if value_type == 'float':
            success = client.set_float('/apps/crebs/' + key_name, value)
        elif value_type == 'int':
            success = client.set_int('/apps/crebs/' + key_name, value)
        elif value_type == 'string':
            success = client.set_string('/apps/crebs/' + key_name, value)
        elif value_type == 'bool':
            success = client.set_bool('/apps/crebs/' + key_name, value)
        else:
            # invalid value_type
            return False
    except:
        # error encountered
        return False
    else:
        # key-set attempted
        return success


def get_gconf_value(value_type, key_name, default_value, set_key = False):
    """Gets a specific value from GConf, creating the key if requested."""

    # Connect to GConf
    try:
        client = gconf.client_get_default()
    except:
        # unable to communicate with GConf
        return default_value
    else:
        # Get value
        try:
            if value_type == 'float':
                v = client.get_float('/apps/crebs/' + key_name)
            elif value_type == 'int':
                v = client.get_int('/apps/crebs/' + key_name)
            elif value_type == 'string':
                v = client.get_string('/apps/crebs/' + key_name)
            elif value_type == 'bool':
                v = client.get_bool('/apps/crebs/' + key_name)
            else:
                # invalid value_type requested
                return default_value
        except:
            # unable to read GConf value
            return default_value
        else:
            if v:
                # value read successfully
                return v
            else:
            # v == False, so check key has been set
                try:
                    # attempt to read the key as if it were another type
                    #   only works for unset keys
                    if value_type == 'float':
                        test = client.get_bool('/apps/crebs/' + key_name)
                    elif value_type == 'int':
                        test = client.get_bool('/apps/crebs/' + key_name)
                    elif value_type == 'string':
                        test = client.get_bool('/apps/crebs/' + key_name)
                    elif value_type == 'bool':
                        test = client.get_string('/apps/crebs/' + key_name)
                except:
                    # type mismatch error confirms key is set
                    return v
                else:
                    # Key hasn't been set, so set it if asked
                    if set_key:
                        set_gconf_value(value_type, key_name, default_value)

                    return default_value


def get_defaults_config():
    """Loads configuration from GConf, creating it if it doesn't exist yet."""

    # Define default defaults
    d_default_duration = 900.0
    d_default_transition = 3.0

    d_directories_images = os.getenv('HOME')
    d_directories_slideshows = os.path.join(os.getenv('HOME'), '.crebs')

    d_files_standard_output = False
    d_files_create_files = True
    d_files_force_overwrite = False

    d_wallpaper_register_wallpaper = True
    d_wallpaper_set_wallpaper = True

    d_notify_send_notification = True

    # Look for GConf values
    default_duration = get_gconf_value('float', 'times/default_duration', d_default_duration, True)
    default_transition = get_gconf_value('float', 'times/default_transition', d_default_transition, True)

    directories_images = get_gconf_value('string', 'directories/images', d_directories_images, True)
    directories_slideshows = get_gconf_value('string', 'directories/slideshows', d_directories_slideshows, True)

    files_standard_output = get_gconf_value('bool', 'files/standard_output', d_files_standard_output, True)
    files_create_files = get_gconf_value('bool', 'files/create_files', d_files_create_files, True)
    files_force_overwrite = get_gconf_value('bool', 'files/force_overwrite', d_files_force_overwrite, True)

    wallpaper_register_wallpaper = get_gconf_value('bool', 'wallpaper/register_wallpaper', d_wallpaper_register_wallpaper, True)
    wallpaper_set_wallpaper = get_gconf_value('bool', 'wallpaper/set_wallpaper', d_wallpaper_set_wallpaper, True)

    notify_send_notification = get_gconf_value('bool', 'notify/send_notification', d_notify_send_notification, True)

    # Validate values
    if default_duration < 0 or default_duration > 12960000:
        if default_duration < 0:
            default_duration = 0
        else:
            default_duration = 12960000

        set_gconf_value('float', 'times/default_duration', default_duration)

    if default_transition < 0 or default_transition > 86400:
        if default_transition < 0:
            default_transition = 0
        else:
            default_transition = 86400

        set_gconf_value('float', 'times/default_transition', default_transition)

    if not os.path.exists(directories_images):
        directories_images = d_directories_images
        set_gconf_value('string', 'directories/images', directories_images)

    # Package and return
    d = { \
        'default_duration'      : default_duration, \
        'default_transition'    : default_transition, \
        'images_directory'      : directories_images, \
        'slideshows_directory'  : directories_slideshows, \
        'standard_output'       : files_standard_output, \
        'file_creation'         : files_create_files, \
        'force_overwrite'       : files_force_overwrite, \
        'register_wallpaper'    : wallpaper_register_wallpaper, \
        'set_wallpaper'         : wallpaper_set_wallpaper, \
        'notify'                : notify_send_notification \
        }

    return d

