#!/usr/bin/env python

import os
from os.path import join, exists, isdir, basename
import urllib

try:
    import gtk
except:
    sys.exit(1)

from crebs.times        import get_scaled_duration
from crebs.filesystem   import safe_filename, make_needed_dirs
from crebs.imagelists   import create_gui_image_list, convert_to_gui_image_list
from crebs.slideshows   import is_slideshow_file, import_slideshow, create_slideshow_xml, write_slideshow
from crebs.wallpapers   import register_wallpaper, set_wallpaper
from crebs.notify       import send_notification
from crebs.debug        import report


class CrebsGui:
    """Creates the GUI."""

    def __init__(self, files, options):
        """Sets up default data and GUI."""

        report(3, "Creating CreBS GUI class...")

        # Terminal conditions
        if options.output or options.filename:
            self.close_afterwards = True
        else:
            self.close_afterwards = False

        # Timings
        self.scales = (1, 60, 3600)
        self.default_duration = options.default_duration
        self.default_transition = options.default_transition
        (self.default_duration_scaled, self.default_scale) = get_scaled_duration(self.default_duration)

        # Copy options to GUI
        self.verbosity          = options.verbosity
        self.name               = options.name
        self.register_wallpaper = options.register_wallpaper
        self.set_wallpaper      = options.set_wallpaper
        self.directory          = options.directory
        self.filename           = options.filename
        self.output             = options.output
        self.force_overwrite    = options.force_overwrite
        self.standard_output    = options.standard_output
        self.file_creation      = options.file_creation
        self.notify             = options.notify

        # Image list: filename; icon; use def. dur.; dur./scale; scale; use def. trans.; trans
        self.gui_image_list = create_gui_image_list(files, options)

        # Set up GUI
        b = gtk.Builder()

        b.add_from_file(join(GLADE_DIR, 'create-background-slideshow.glade'))
        b.add_from_file(join(GLADE_DIR, 'add-images.glade'))
        b.add_from_file(join(GLADE_DIR, 'clear-images.glade'))
        b.add_from_file(join(GLADE_DIR, 'file-overwrite.glade'))

        # Connect to GUI signals
        b.connect_signals({
            'on_wndMain_destroy'    : gtk.main_quit,
            "on_drop"               : self.add_by_drop,

            "on_select"             : self.on_select_image,
            "on_unselect"           : self.on_unselect_image,
            "on_iconview_keyboard"  : self.icvImages_keyboard,

            "on_add"                : self.show_add_dialog,
            "on_copy"               : self.copy_image,
            "on_earlier"            : self.move_image_earlier,
            "on_later"              : self.move_image_later,
            "on_remove"             : self.remove_image,
            "on_clear"              : self.clear_images,

            "on_new_def_time"       : self.change_default_duration,
            "on_new_def_scale"      : self.change_default_timescale,
            "on_new_def_trans"      : self.change_default_transition,

            "on_new_img_time"       : self.change_selected_duration,
            "on_new_img_scale"      : self.change_selected_timescale,
            "on_new_img_trans"      : self.change_selected_transition,

            "on_toggle_time"        : self.toggle_time,
            "on_toggle_trans"       : self.toggle_trans,

            "on_named"              : self.name_changed,
            "on_enter"              : self.try_to_save,
            "on_save"               : self.save_slideshow,

            "on_add_images"         : self.on_add_images,
            "on_cancel"             : self.close_add_dialog
            })

        # Fetch GUI objects
        ### Main CreBS window
        self.wndMain            = b.get_object("wndMain")
        self.icvImages          = b.get_object("icvImages")
        self.btnAdd             = b.get_object("btnAdd")
        self.btnCopy            = b.get_object("btnCopy")
        self.btnEarlier         = b.get_object("btnEarlier")
        self.btnLater           = b.get_object("btnLater")
        self.btnRemove          = b.get_object("btnRemove")
        self.btnClear           = b.get_object("btnClear")

        self.nbkTimings         = b.get_object("nbkTimings")

        self.lblSelected        = b.get_object("lblSpecifics")
        self.lblDurSelected     = b.get_object("lblDisplaySpecific")
        self.lblTrnSelected     = b.get_object("lblTransitionSpecific")

        self.spnDurSelected     = b.get_object("spnTimeSpecific")
        self.spnTrnSelected     = b.get_object("spnTransSpecific")
        self.DurDefault         = b.get_object("spnTimeDefault").get_adjustment()
        self.TrnDefault         = b.get_object("spnTransDefault").get_adjustment()
        self.DurSelected                    = self.spnDurSelected.get_adjustment()
        self.TrnSelected                    = self.spnTrnSelected.get_adjustment()

        self.cmbDurScaleDefault = b.get_object("cmbScaleTimeDefault")
        self.cmbDurScaleSelected = b.get_object("cmbScaleTimeSpecific")
        self.lstScalesDefault   = b.get_object("lstScalesDefault")
        self.lstScalesSelected  = b.get_object("lstScalesSpecific")

        self.chkUseDefDur       = b.get_object("chkUseDefDisplay")
        self.chkUseDefTrn       = b.get_object("chkUseDefTransition")

        self.lblSecondsDefault  = b.get_object("lblSecondsDefault")
        self.lblSecondsSelected = b.get_object("lblSecondsSpecific")

        self.lblName            = b.get_object("lblName")
        self.txtName            = b.get_object("txtName")
        self.Name                        = self.txtName.get_text
        self.btnSave            = b.get_object("btnSave")

        ### 'Add Images' dialog
        self.dlgAddImages       = b.get_object("dlgAddImages")

        ### 'Clear Images' dialog
        self.dlgConfirmClear    = b.get_object("dlgConfirmClear")
        self.btnKeep            = b.get_object("btnKeep")

        ### 'File Overwrite' dialog
        self.dlgOverwrite       = b.get_object("dlgConfirmOverwrite")
        self.btnDontOverwrite   = b.get_object("btnDontOverwrite")

        # Set window as drag-drop destination
        self.wndMain.drag_dest_set( gtk.DEST_DEFAULT_MOTION |
                                    gtk.DEST_DEFAULT_HIGHLIGHT |
                                    gtk.DEST_DEFAULT_DROP,
                                    [ ( 'text/uri-list', 0, 80 ) ],
                                    gtk.gdk.ACTION_COPY )

        # 'Add Images' dialog: file filters and directory
        self.addFileFiltersToAddDialog()
        self.add_images_dir = options.images_directory

        # Add images to the icon-view widget
        self.icvImages.set_model(self.gui_image_list)
        self.icvImages.set_pixbuf_column(1)
        self.icvImages.set_tooltip_column(0)

        # Set GUI data
        self.DurDefault.set_all (self.default_duration_scaled, 0, 3600, 1, 0, 0)
        self.DurSelected.set_all(self.default_duration_scaled, 0, 3600, 1, 0, 0)
        self.cmbDurScaleDefault.set_active (self.default_scale)
        self.cmbDurScaleSelected.set_active(self.default_scale)
        self.TrnDefault.set_all (self.default_transition, 0, 86400, 1, 0, 0)
        self.TrnSelected.set_all(self.default_transition, 0, 86400, 1, 0, 0)

        if options.name:
            self.txtName.set_text(options.name)

        # Set inactive widgets and tooltips
        self.use_gui_for_name = True
        if not options.file_creation:
            self.lblName.set_sensitive(False)
            self.txtName.set_sensitive(False)
            self.use_gui_for_name = False
            self.btnSave.set_tooltip_text(_("Create this slideshow."))

            if options.standard_output:
                self.txtName.set_text(_("Standard Output only"))
            else:
                self.txtName.set_text(_("No files will be created."))

        elif not options.register_wallpaper:
            if options.filename or options.output:
                self.lblName.set_sensitive(False)
                self.txtName.set_sensitive(False)
                self.use_gui_for_name = False
                if options.output:
                    self.txtName.set_text(options.output)
                else:
                    self.txtName.set_text(options.filename)
                self.btnSave.set_tooltip_text(_("Save this slideshow and exit."))
            else:
                self.btnSave.set_tooltip_text(_("Save this slideshow."))

        elif not options.set_wallpaper:
            if options.filename or options.output:
                self.lblName.set_sensitive(False)
                self.txtName.set_sensitive(False)
                self.use_gui_for_name = False
                if options.output:
                    self.txtName.set_text(options.output)
                else:
                    self.txtName.set_text(options.filename)
                self.btnSave.set_tooltip_text(_("Register this slideshow and exit."))
            else:
                self.btnSave.set_tooltip_text(_("Register this slideshow."))

        self.on_unselect_image()
        self.check_if_clearable()
        self.check_if_saveable()
        self.select_page('DEFAULTS')

        report(3, "Success.", 0.5)


    def show(self):
        """Displays the GUI and starts the GTK Main Loop."""

        self.wndMain.set_focus(self.btnAdd)
        self.wndMain.show()
        gtk.main()


    def addFileFiltersToAddDialog(self):
        """Adds image and slideshow file filters to 'Add Images' dialog."""

        fil_img = gtk.FileFilter()
        fil_img.set_name(_("Images"))
        fil_img.add_mime_type("image/jpeg")
        fil_img.add_mime_type("image/png")
        fil_img.add_mime_type("image/gif")
        fil_img.add_mime_type("image/svg")
        fil_img.add_mime_type("image/svg+xml")

        fil_bg = gtk.FileFilter()
        fil_bg.set_name(_("Slideshows"))
        fil_bg.add_custom(gtk.FILE_FILTER_FILENAME, is_slideshow_file)

        fil_all = gtk.FileFilter()
        fil_all.set_name(_("All supported types"))
        fil_all.add_mime_type("image/jpeg")
        fil_all.add_mime_type("image/png")
        fil_all.add_mime_type("image/gif")
        fil_all.add_mime_type("image/svg")
        fil_all.add_mime_type("image/svg+xml")
        fil_all.add_custom(gtk.FILE_FILTER_FILENAME, is_slideshow_file)

        self.dlgAddImages.add_filter(fil_all)
        self.dlgAddImages.add_filter(fil_img)
        self.dlgAddImages.add_filter(fil_bg)


    def select_page(self, page):
        """Selects a page in the Timings notebook"""

        if page == 'SELECTED':
            self.nbkTimings.set_current_page(1)
        elif page == 'DEFAULTS':
            self.nbkTimings.set_current_page(0)
        else:
            report(0, 'Programmer Error: Invalid page name')


    def check_if_clearable(self):
        """Checks if there are any images to clear."""

        if (len(self.gui_image_list) == 0):
            self.btnClear.set_sensitive(False)
            self.select_page('DEFAULTS')
        else:
            self.btnClear.set_sensitive(True)


    def check_if_saveable(self):
        """Checks there are at least two images, and has a name."""

        if (len(self.gui_image_list) >= 2 and self.Name() != ""):
            self.btnSave.set_sensitive(True)
            return True
        else:
            self.btnSave.set_sensitive(False)
            return False


    def get_selection(self):
        """Returns the position (first=0) of the selected image."""

        if (len(self.icvImages.get_selected_items()) == 0):
            return -1
        else:
            return self.icvImages.get_selected_items()[0][0]


    def is_selected_image_first(self):
        """Returns True if the selected image is first in the slideshow."""

        if (self.get_selection() == 0):
            return True


    def is_selected_image_last(self):
        """Returns True if the selected image is last in the slideshow."""

        if (self.get_selection() + 1 == len(self.gui_image_list)):
            return True
        else:
            return False


    def activate_movement_section(self, Active):
        """Sets the sensitivity of the movement arrows."""

        self.btnRemove.set_sensitive(Active)

        if (Active == True):
            if (self.is_selected_image_first()):
                self.btnEarlier.set_sensitive(False)
            else:
                self.btnEarlier.set_sensitive(True)

            if (self.is_selected_image_last()):
                self.btnLater.set_sensitive(False)
            else:
                self.btnLater.set_sensitive(True)
        else:
            self.btnEarlier.set_sensitive(False)
            self.btnLater.set_sensitive(False)


    def activate_selection_section(self, Active):
        """Sets the sensitivity of the 'Selected Image' section."""

        self.lblSelected.set_sensitive(Active)
        self.lblDurSelected.set_sensitive(Active)

        self.chkUseDefDur.set_sensitive(Active)

        if (self.chkUseDefDur.get_active()):
            self.spnDurSelected.set_sensitive(False)
            self.cmbDurScaleSelected.set_sensitive(False)
        else:
            self.spnDurSelected.set_sensitive(Active)
            self.cmbDurScaleSelected.set_sensitive(Active)

        self.lblTrnSelected.set_sensitive(Active)
        self.chkUseDefTrn.set_sensitive(Active)

        if (self.chkUseDefTrn.get_active()):
            self.spnTrnSelected.set_sensitive(False)
            self.lblSecondsSelected.set_sensitive(False)
        else:
            self.spnTrnSelected.set_sensitive(Active)
            self.lblSecondsSelected.set_sensitive(Active)


    def on_select_image(self, widget = 'allow-non-widget-callers'):
        """An image from the slideshow has been selected."""

        if (self.get_selection() < 0 or len(self.gui_image_list) == 0):
            self.on_unselect_image()
        else:
            self.btnCopy.set_sensitive(True)
            self.activate_movement_section(True)
            self.activate_selection_section(True)
            self.select_page('SELECTED')

	    # Update selection section with image timings/defaultness.
            if (self.gui_image_list[self.get_selection()][2]):
                self.chkUseDefDur.set_active(True)
                self.DurSelected.set_value(self.DurDefault.get_value())
                self.cmbDurScaleSelected.set_active(self.cmbDurScaleDefault.get_active())
            else:
                self.chkUseDefDur.set_active(False)
                self.DurSelected.set_value(self.gui_image_list[self.get_selection()][3])
                self.cmbDurScaleSelected.set_active(self.gui_image_list[self.get_selection()][4])

            if (self.gui_image_list[self.get_selection()][5]):
                self.chkUseDefTrn.set_active(True)
                self.TrnSelected.set_value(self.TrnDefault.get_value())
            else:
                self.chkUseDefTrn.set_active(False)
                self.TrnSelected.set_value(self.gui_image_list[self.get_selection()][6])


    def on_unselect_image(self):
        """Inactivates widgets when no images are selected."""

        self.btnCopy.set_sensitive(False)
        self.activate_movement_section(False)
        self.activate_selection_section(False)

        self.chkUseDefDur.set_active(True)
        self.DurSelected.set_value(self.DurDefault.get_value())
        self.cmbDurScaleSelected.set_active(self.cmbDurScaleDefault.get_active())
        self.chkUseDefTrn.set_active(True)
        self.TrnSelected.set_value(self.TrnDefault.get_value())


    def add_images_to_gui(self, filenames, indent_offset = 0):
        """Adds selected images to GUI image list."""

        image_list = []

        prev_img_count = len(self.gui_image_list)

        for filename in filenames:
            if not is_slideshow_file(filename, True, indent_offset):
                image_list.append((filename, self.default_duration, self.default_transition))

            else:
                slides = import_slideshow(filename, indent_offset)
                for slide in slides:
                    image_list.append(slide)

        self.gui_image_list = convert_to_gui_image_list(image_list, self.default_duration, self.default_transition, self.gui_image_list)

        # Select the first of the new images
        if len(self.gui_image_list) > prev_img_count:
            self.icvImages.select_path(prev_img_count)

        self.check_if_saveable()
        self.check_if_clearable()


    def add_by_drop(self, widget, context, x, y, selection, target_type, timestamp):
        """Add an image by dropping it onto the window."""

        report(3, "File(s) dropped onto main window.")

        if target_type == 80:
            report(3, "Processing dropped files...")
            filenames=[]
            path_list = selection.data.strip('\r\n\x00').split()
            for path in path_list:
                if path.startswith('file://'):
                    path = path[7:]
                elif path.startswith('file:'):
                    path = path[5:]
                # Fix characters escaped by Nautilus, e.g. %20 -> space
                path = urllib.url2pathname(path)
                filenames.append(path)

            self.add_images_to_gui(filenames, 1)
            report(3, "Success.", 0.5)


    def show_add_dialog(self, widget):
        """Shows a file-chooser dialog to add images."""

        self.dlgAddImages.set_current_folder(self.add_images_dir)
        self.dlgAddImages.show()


    def close_add_dialog(self, widget):
        """Closes the file-chooser dialog after adding images."""

        self.dlgAddImages.hide()
        return True


    def on_add_images(self, widget):
        "Adds images after being chosen in the 'Add Images' dialog."""

        filenames = self.dlgAddImages.get_filenames()

        # Check for a directory to open instead
        for filename in filenames:
            if isdir(filename):
                self.dlgAddImages.set_current_folder(filename)
                return False

        self.add_images_dir = self.dlgAddImages.get_current_folder()
        self.dlgAddImages.hide()

        self.add_images_to_gui(filenames)


    def copy_image(self, widget):
        """Copies the selected image."""

        sel = self.get_selection()
        report(2, _("Copying image at position {0}...").format(str(sel)), 0)
        report(2, self.gui_image_list[sel][0], 1)

        sib = self.gui_image_list.get_iter_from_string(str(sel))
        src = self.gui_image_list[sel]
        self.gui_image_list.insert_after(sib, src)

        report(3, "Slideshow now has {0} image(s).".format(str(len(self.gui_image_list))), 1)
        report(3, "Success.", 0.5)

        self.icvImages.select_path(sel + 1)
        self.on_select_image()
        self.check_if_saveable()


    def move_selected_image(self, relative_movement):
        """Moves the selected image by the specified amount."""

        current_position = self.get_selection()
        a = self.gui_image_list.get_iter_from_string(str(current_position))
        b = self.gui_image_list.get_iter_from_string(str(current_position + relative_movement))
        self.gui_image_list.swap(a, b)

        self.on_select_image()


    def move_image_earlier(self, widget):
        """Move selected image before the previous image in the slideshow."""
        self.move_selected_image(-1)


    def move_image_later(self, widget):
        """Move selected image behind the next image in the slideshow."""
        self.move_selected_image(1)


    def remove_image(self, widget, img=-1):
        """Removes the selected image from the slideshow."""

        if (img == -1):
            sel = self.get_selection()
        else:
            sel = img

        going_pos = str(sel)
        going_path = self.gui_image_list[sel][0]

        report(2, _("Removing image at position {0}...").format(going_pos))
        report(2, going_path, 1)

        self.gui_image_list.remove(self.gui_image_list.get_iter_from_string(going_pos))
        report(3, "Slideshow now has " + str(len(self.gui_image_list)) + " image(s).", 1)
        report(3, "Success.", 0.5)

        if (sel < len(self.gui_image_list)):
            self.icvImages.select_path(sel)
        else:
            if (len(self.gui_image_list) > 0):
                self.icvImages.select_path(sel - 1)

        self.check_if_saveable()
        self.check_if_clearable()


    def clear_images(self, widget):
        """Shows a confirmation dialogue."""

        self.dlgConfirmClear.set_focus(self.btnKeep)
        self.dlgConfirmClear.show_all()
        result = self.dlgConfirmClear.run()
        self.dlgConfirmClear.hide()

        if result == 28:
            report(2, _("Clearing all images..."))
            self.gui_image_list.clear()

            self.check_if_saveable()
            self.check_if_clearable()
            report(3, "Slideshow now has {0} image(s).".format(str(len(self.gui_image_list))), 1)
            report(3, "Success.", 0.5)
            

    def icvImages_keyboard(self, widget, event):
        """Handles (crudely) keyboard interaction with the IconView widget."""

        if (str(event)[-7:-1] == "Delete"):
            report(3, "Delete key pressed while icons view active.")

            sel = self.get_selection()
            if (sel >= 0):
                self.remove_image(widget, sel)
                

    def change_default_duration(self, widget):
        """Responds to changing the default duration."""

        self.default_duration_scaled = self.DurDefault.get_value()
        self.default_duration = self.default_duration_scaled * self.scales[self.default_scale]

        n = self.DurDefault.get_value()
        self.lstScalesDefault.set_value(self.lstScalesDefault.get_iter_from_string("0"), 0, N_("second", "seconds", n))
        self.lstScalesDefault.set_value(self.lstScalesDefault.get_iter_from_string("1"), 0, N_("minute", "minutes", n))
        self.lstScalesDefault.set_value(self.lstScalesDefault.get_iter_from_string("2"), 0, N_("hour", "hours", n))

        if (self.chkUseDefDur.get_active()):
            self.DurSelected.set_value(self.DurDefault.get_value())


    def change_selected_duration(self, widget):
        """Responds to changing the duration for the selected image."""

        img_pos = self.get_selection()
        if (img_pos > -1):
            self.gui_image_list.set_value(self.gui_image_list.get_iter_from_string(str(img_pos)), 3, float(self.DurSelected.get_value()))

        n = self.DurSelected.get_value()
        self.lstScalesSelected.set_value(self.lstScalesSelected.get_iter_from_string("0"), 0, N_("second", "seconds", n))
        self.lstScalesSelected.set_value(self.lstScalesSelected.get_iter_from_string("1"), 0, N_("minute", "minutes", n))
        self.lstScalesSelected.set_value(self.lstScalesSelected.get_iter_from_string("2"), 0, N_("hour", "hours", n))

    def change_default_timescale(self, widget):
        """Responds to changing the default duration."""

        self.default_scale = self.cmbDurScaleDefault.get_active()
        self.default_duration = self.default_duration_scaled * self.scales[self.default_scale]

        if (self.chkUseDefDur.get_active()):
            self.cmbDurScaleSelected.set_active(self.cmbDurScaleDefault.get_active())


    def change_selected_timescale(self, widget):
        """Responds to changing the timescale for the selected image."""

        img_pos = self.get_selection()
        if (img_pos > -1):
            self.gui_image_list.set_value(self.gui_image_list.get_iter_from_string(str(img_pos)), 4, int(self.cmbDurScaleSelected.get_active()))


    def change_default_transition(self, widget):
        """Responds to changing the default transition."""

        self.default_transition = self.TrnDefault.get_value()

        if (self.chkUseDefTrn.get_active()):
            self.TrnSelected.set_value(self.TrnDefault.get_value())
            self.lblSecondsSelected.set_text(self.lblSecondsDefault.get_text())


    def change_selected_transition(self, widget):
        """Responds to changing the transition time for the selected image."""

        img_pos = self.get_selection()
        if (img_pos > -1):
            self.gui_image_list.set_value(self.gui_image_list.get_iter_from_string(str(img_pos)), 6, float(self.TrnSelected.get_value()))


    def toggle_time(self, widget):
        """Responds to toggling the 'Use default display time' checkbox."""

        img_pos = self.get_selection()

        if (self.chkUseDefDur.get_active()):
            if (img_pos > -1):
                self.gui_image_list.set_value(self.gui_image_list.get_iter_from_string(str(img_pos)), 2, True)
                self.gui_image_list.set_value(self.gui_image_list.get_iter_from_string(str(img_pos)), 3, float(self.DurDefault.get_value()))
                self.gui_image_list.set_value(self.gui_image_list.get_iter_from_string(str(img_pos)), 4, int(self.cmbDurScaleDefault.get_active()))
                self.spnDurSelected.set_value(self.gui_image_list[img_pos][3])
                self.cmbDurScaleSelected.set_active(self.gui_image_list[img_pos][4])

            self.spnDurSelected.set_sensitive(False)
            self.cmbDurScaleSelected.set_sensitive(False)
        else:
            if (img_pos > -1):
                self.gui_image_list.set_value(self.gui_image_list.get_iter_from_string(str(img_pos)), 2, False)

            self.spnDurSelected.set_sensitive(True)
            self.cmbDurScaleSelected.set_sensitive(True)


    def toggle_trans(self, widget):
        """Responds to toggling the 'Use default transition time' checkbox."""

        img_pos = self.get_selection()

        if (self.chkUseDefTrn.get_active()):
            if (img_pos > -1):
                self.gui_image_list.set_value(self.gui_image_list.get_iter_from_string(str(img_pos)), 5, True)
                self.gui_image_list.set_value(self.gui_image_list.get_iter_from_string(str(img_pos)), 6, float(self.TrnDefault.get_value()))
                self.spnTrnSelected.set_value(self.gui_image_list[img_pos][6])

            self.spnTrnSelected.set_sensitive(False)
            self.lblSecondsSelected.set_sensitive(False)
        else:
            if (img_pos > -1):
                self.gui_image_list.set_value(self.gui_image_list.get_iter_from_string(str(img_pos)), 5, False)

            self.spnTrnSelected.set_sensitive(True)
            self.lblSecondsSelected.set_sensitive(True)


    def name_changed(self, widget):
        """Responds to changing the name in the GUI."""

        self.check_if_saveable()


    def try_to_save(self, widget):
        """Tries to save slideshow after pressing ENTER in Name box."""

        if (self.check_if_saveable() == True):
            self.save_slideshow()


    def warn_file_overwrite(self):
        """Displays a file-overwrite warning dialog, and return True if user allows overwrite."""

        self.dlgOverwrite.set_focus(self.btnDontOverwrite)
        self.dlgOverwrite.show_all()
        result = self.dlgOverwrite.run()
        self.dlgOverwrite.hide()

        if result == 30:
            return True
        else:
            return False


    def save_slideshow(self, widget='dummy'):
        """Saves the slideshow."""

        # Convert GUI image list to basic image list
        image_list = []

        for image in self.gui_image_list:
            filename  = image[0]

            if image[2]:
                duration = self.default_duration
            else:
                duration  = image[3] * self.scales[image[4]]

            if image[5]:
                transition = self.default_transition
            else:
                transition = image[6]

            image_list.append((filename, duration, transition))

        # Create slideshow
        if self.use_gui_for_name:
            s_name = self.Name()
        else:
            s_name = self.name

        if self.output:
            self.filename = basename(self.output)
        else:
            if self.filename:
                self.output = join(self.directory, self.filename)
            elif s_name:
                self.filename = safe_filename(s_name)
                self.output = join(self.directory, self.filename)
            else:
                self.file_creation = False

        s_xml = create_slideshow_xml(image_list, s_name)

        if self.standard_output or self.verbosity >= 5:
            print s_xml

        if self.file_creation:
            # Check for overwrite
            if exists(self.output) and not self.force_overwrite:
                ok_to_write = self.warn_file_overwrite()
            else:
                ok_to_write = True

            # Write slideshow
            if ok_to_write:
                make_needed_dirs(self.output)

                slideshow_written = write_slideshow(self.output, s_xml)

                # Register wallpaper
                if slideshow_written and self.register_wallpaper:
                    register_wallpaper(s_name, self.output)
                    # set_wallpaper not dependent on success

                    # Set wallpaper
                    if self.set_wallpaper:
                        wallpaper_set = set_wallpaper(self.output)

                        # Send notification
                        if wallpaper_set and self.notify:
                            send_notification(s_name)

        if self.close_afterwards:
            gtk.main_quit()
        else:
            # Reset output and filename
            self.output = ''
            self.filename = ''

