#!/usr/bin/env python

import os
from datetime import datetime
import libxml2
import re

from crebs.entities     import decode_entities, encode_xml_entities
from crebs.debug        import report


def add_to_slideshow_xml(current_image, duration, transition, next_image):
    """Creates the XML fragment for an image and its transition."""

    report(3, "Adding an image to the slideshow XML...", 2)
    report(3, current_image, 3)
    report(3, "for {0}, changing over {1} to".format(duration, transition), 3.5)
    report(3, next_image, 3)

    # Make names XML-safe
    current_image = encode_xml_entities(current_image)
    next_image    = encode_xml_entities(next_image)

    duration = round(duration, 1)
    transition = round(transition, 1)

    bg_xml = ''

    if duration:
        bg_xml += """\
  <static>
    <duration>{1}</duration>
    <file>{0}</file>
  </static>
""".format(current_image, duration)

    if transition:
        bg_xml += """\
  <transition>
    <duration>{2}</duration>
    <from>{0}</from>
    <to>{1}</to>
  </transition>

""".format(current_image, next_image, transition)

    elif duration:
        bg_xml += "\n"

    report(3, "Success.", 2.5)

    return bg_xml



def create_slideshow_xml(image_list, s_name=''):
    """Creates a slideshow from the list of images and returns the XML."""

    # image_list format:
    ##  image filename
    ##  image duration
    ##  image transition time

    report(2, _("Creating slideshow XML..."))

    # Open
    report(3, "Creating header...", 1)

    if s_name:
        bg_xml = '<!-- '
        bg_xml += "Name:" + ' '
        bg_xml += encode_xml_entities(s_name)
        bg_xml += " -->\n"
    else:
        bg_xml = ""

    bg_xml += """\
<background>
  <starttime>
    <year>2009</year>
    <month>09</month>
    <day>06</day>
    <hour>00</hour>
    <minute>00</minute>
    <second>00</second>
  </starttime>

"""

    report(3, "Success.", 1.5)

    # Add images
    if len(image_list):
        report(3, "Adding images...", 1)

        current_image = image_list[0][0]
        duration =      image_list[0][1]
        transition =    image_list[0][2]

        for i in range(1, len(image_list)):
            next_image = image_list[i][0]
            bg_xml += add_to_slideshow_xml(current_image, duration, transition, next_image)

            current_image = next_image
            duration =   image_list[i][1]
            transition = image_list[i][2]

        ### Loop back to first image
        bg_xml += add_to_slideshow_xml(current_image, duration, transition, image_list[0][0])

        report(3, "Success.", 1.5)

    # Close
    report(3, "Closing the XML document...", 1)

    bg_xml += "</background>\n<!-- "
    bg_xml += "Created {0} with {1}".format(datetime.now().replace(microsecond=0).isoformat(' '), APP_LONG_NAME)
    bg_xml += " -->"

    report(3, "Success.", 1.5)

    report(3, "Success.", 0.5)

    return bg_xml


def import_slideshow(s_filename, indent_offset = 0):
    """Imports a slideshow, including its images and timings."""

    report(2, _("Importing a slideshow..."), indent_offset)
    report(2, s_filename, indent_offset + 1)

    slideshow = []

    try:
        slides = libxml2.parseFile(s_filename)
    except IOError:
        report (2, _("Failure: Unable to open and parse the slideshow file."), indent_offset + 0.5)
    else:
        root = slides.getRootElement()

        if root.name == 'background':

            # Initialise variables
            filename = ''
            duration = 0
            transition = 0

            # Fetch static and transition elements
            bg_elements = root.children

            for bgl in bg_elements:
                if bgl.name == 'static':
                    # This is a new image; append the old one
                    if filename:
                        slideshow.append((filename, duration, transition))
                        report(3, filename, indent_offset + 1)

                    # Re-initialise image variables
                    filename = ''
                    duration = 0
                    transition = 0

                    # Get new filename and duration
                    st_elements = bgl.children
                    for stl in st_elements:
                        if stl.name == 'file':
                            filename = decode_entities(str(stl.children))
                        elif stl.name == 'duration':
                            duration = float(str(stl.children))

                elif bgl.name == 'transition':
                    # Re-initialise transition variables
                    tr_filename = ''
                    tr_duration = 0

                    # Remember filename and transition
                    tr_elements = bgl.children
                    for trl in tr_elements:
                        if trl.name == 'from':
                            tr_filename = decode_entities(str(trl.children))
                        elif trl.name == 'duration':
                            tr_duration = float(str(trl.children))

                    # Check for matching static element
                    if not tr_filename == filename:
                        # This is a new image; append the old one
                        if filename:
                            slideshow.append((filename, duration, transition))
                            report(3, filename, indent_offset + 1)

                        # Store new data
                        filename = tr_filename
                        duration = 0

                    transition = tr_duration

            # Append final image
            if filename:
                slideshow.append((filename, duration, transition))
                report(3, filename, indent_offset + 1)

            # Check for success
            if len(slideshow) < 1:
                report(2, _("Failure: No images were found."), indent_offset + 0.5)
            else:
                report(3, "Success.", indent_offset + 0.5)

        return slideshow


def is_slideshow_file(flags, direct = False, indent_offset = 0):
    """Returns True if the 'potential' file is probably a valid wallpaper slideshow."""

    is_slideshow = False

    if direct:
        potential = flags
    else:
        potential = flags[0]

    report(3, "Checking whether {0} is a slideshow file...".format(potential), indent_offset)

    try:
        f = open(potential, 'r')
        try:
            filestart = f.read(1024)
        finally:
            f.close()
    except IOError:
        if os.path.isdir(potential):
            report(3, "No: It's a directory.", indent_offset + 0.5)
        else:
            report(3, "Error: Cannot read from potential background file (" + potential + ").", indent_offset + 0.5)
    else:
        if  re.search('background', filestart) \
          and \
            re.search('starttime', filestart) \
          and \
            re.search('year', filestart) \
          and \
            re.search('month', filestart) \
          and \
            re.search('day', filestart):

            is_slideshow = True
            report(3, "Yes: This is probably a slideshow.", indent_offset + 0.5)

        else:
            report(3, "No: This probably isn't a slideshow.", indent_offset + 0.5)

    return is_slideshow


def write_slideshow(s_filename, s_xml):
    """Saves the slideshow."""

    report(2, _("Writing slideshow XML to file..."))

    success = False

    try:
        f = open(s_filename, 'w')
        try:
            f.write(s_xml)
            success = True
            report(3, "Success.", 0.5)
        finally:
            f.close()
    except IOError:
        report(2, _("Failure: Unable to create slideshow file."), 0.5)
        report(2, _("Error: Unable to create slideshow file. Ensure you have write access for {0}").format(s_filename), 0.5, True)

    return success

