#!/usr/bin/env python

import os
from os.path            import join, exists
import gtk
from hashlib            import md5

from crebs.filesystem   import make_needed_dirs
from crebs.debug        import report


def resize_thumbnail(pixbuf, target_size = 96, filename = ''):
    """Resizes a thumbnail and, optionally, saves it for later use."""

    aspect_ratio = float(pixbuf.get_width()) / float(pixbuf.get_height())

    if (aspect_ratio >= 1):
        x = target_size
        y = int(target_size / aspect_ratio)
    else:
        y = target_size
        x = int(target_size * aspect_ratio)

    pixbuf = pixbuf.scale_simple(x,y,gtk.gdk.INTERP_BILINEAR)

    if filename:
        make_needed_dirs(filename)
        try:
            pixbuf.save(filename, 'png')
        except:
            report(4, "Unable to save image to .thumbnails cache.", 2)
        else:
            report(4, "Thumbnail cached.", 2)

    return pixbuf


def get_thumbnail(filename, target_size = 96):
    """Returns a thumbnail for the specified image as a Pixbuf."""

    report(4, "Generating thumbnail...", 1)

    # Validate filename as a usable image type
    try:
        f = open(filename, 'r')
        try:
            twisted_filestarter = f.read(512)
        finally:
            f.close()
    except IOError:
        report(4, "Error: Cannot read as a file: {0}".format(filename), 1.5)
        return False
    else:
        if twisted_filestarter[:8] == "\x89\x50\x4e\x47\x0D\x0A\x1A\x0A":
            report(4, "PNG: {0}".format(filename), 2)

        elif twisted_filestarter[:2] == "\xff\xd8":
            report(4, "JPEG: {0}".format(filename), 2)

        elif twisted_filestarter[:6] == 'GIF89a' or twisted_filestarter[:6] == 'GIF87a':
            report(4, "GIF: {0}".format(filename), 2)

        elif 'xml' in twisted_filestarter and 'http://www.w3.org/2000/svg' in twisted_filestarter:
            report(4, "SVG: {0}".format(filename), 2)

        else:
            report(4, "Invalid filetype: {0}".format(filename), 2)
            report(6, twisted_filestarter) # DEBUG data for -vvvvvv
            return False

    # Look in thumbnail cache first (~/.thumbnails/normal)
    thumb_dir = join(os.getenv('HOME'), '.thumbnails', 'normal')
    thumb = join(thumb_dir, md5('file://' + filename).hexdigest() + '.png')

    if not exists(thumb):
        thumbnail_needed = True
    else:
        thumbnail_needed = False
        filename = thumb
        report(4, "Cached thumbnail located.", 1.5)

    # Generate correctly sized thumbnail
    try:
        pixbuf = gtk.gdk.pixbuf_new_from_file(filename)
    except:
        report(4, "Unable to create pixbuf.", 1.5)
        return False
    else:
        if thumbnail_needed:
            # Save 128-pixel thumbnail to cache
            pixbuf = resize_thumbnail(pixbuf, 128, thumb)

        resized = resize_thumbnail(pixbuf, target_size)

        return resized

